/*****************************************************************************

FILE
  __start_p1.c
  $Date: 2011/05/02 12:26:46 $
  $Revision: 1.7 $

DESCRIPTION
  A minimal startup function for the second core of a MPC56xx family processor.

  This function initializes the stack to the values defined in the LCF file and
  uses the linker defined _SDA_BASE_ and _SDA2_BASE_ symbols to initialize
  r2 and r13 per (EABI). It then calls main_p1().

*/
#include <__ppc_eabi_init.h>
#include <__ppc_eabi_linker.h>	/* linker-generated symbol declarations */

/* These symbols are defined in the Linker configuration file, but are not in */
/* __ppc_eabi_linker.h, so we'll provide the declarations here.  Note that    */
/* _heap_*_p1 definitions are currently not being used.                       */ 
__declspec(section ".init") extern char	_stack_addr_p1[];
__declspec(section ".init") extern char _stack_end_p1[];
__declspec(section ".init") extern char	_heap_addr_p1[];
__declspec(section ".init") extern char _heap_end_p1[];

extern void main_p1();

#ifdef __cplusplus
extern "C" {
#endif

extern __asm void __initMMU(void);
#pragma function_align 8
__declspec(section ".init") asm extern void __start_p1(void)
{
	nofralloc

	/* Initialize MMU */
#if ROM_VERSION
	mflr     r26;
	bl __initMMU;
	mtlr r26;
#else
	lis     r3, 0x1000
    mtMAS0  r3

    lis     r4, 0xC000
    ori     r4, r4, 0x0B00
    mtMAS1  r4

#if __option(vle) 
    lis     r5, 0x0000
    ori     r5, r5, 0x0020
#else
    lis     r5, 0x0000
    ori     r5, r5, 0x0000
#endif
    mtMAS2  r5

    lis     r6, 0x0000
    ori     r6, r6, 0x003f
    mtMAS3  r6

    msync
    tlbwe
    isync
#endif

	/* Initialize stack pointer	*/
	lis	r1, _stack_addr_p1@ha
	/* Emulate a calling function's frame, for correct LR save in main_p1 */
	addi	r1, r1, (_stack_addr_p1@l) - 16

	/* Note that we are assuming small data is shared between the cores */
	/* Initialize small data area pointers (EABI)                       */
	lis   r2, _SDA2_BASE_@ha
	addi  r2, r2, _SDA2_BASE_@l
  
	lis   r13, _SDA_BASE_@ha
	addi  r13, r13, _SDA_BASE_@l

	/* now call z0's main program (using short branch)... */ 
	bl main_p1
	
	
  	/* if main returns, just hang here */
here:   b here

	blr
}


#ifdef __cplusplus
}
#endif

